#!/usr/bin/env ruby
#encoding=utf-8
# 服务器主从设置脚本
# 用例： ./config_server <role> <server1.com的IP> <server2.com的IP> <haserver.com的IP>
# role：可选项为master或slave。
# IP地址说明：server1.com和server2.com为实际两台服务器的IP;haserver.com为需要配置的虚拟IP，虚拟IP不能被占用，需要和实际服务器IP在同一网段。
#

def put_usage
  puts "格式：config_server <role> <server1.com的IP> <server2.com的IP> <haserver.com的IP> "
  puts "role：指定该服务器为主机或备机。可选项为master或slave。"
  puts "IP地址说明：server1.com和server2.com为实际两台服务器的IP;haserver.com为需要配置的虚拟IP，虚拟IP不能被占用，需要和实际服务器IP在同一网段。"
  puts "执行本脚本之前请先配置网络和rails环境。请先在master上执行，再在slave上执行。"
  puts "具体哪台是master，哪台是slave没有关系。"
  puts "例如：./config_server master 192.168.100.216 192.168.100.217 192.168.100.236"
end

def set_hosts server1, server2, haserver, role
  puts "设置hosts"
  another_server = 
    case role
    when "master"
      server2
    when "slave"
      server1
    end

  cmd1 = 'sed -i "s/[0-9]\{1,3\}.[0-9]\{1,3\}.[0-9]\{1,3\}.[0-9]\{1,3\}\([ \t]\+server1.com\)/' + server1 + '\1/g" /etc/hosts'
  cmd2 = 'sed -i "s/[0-9]\{1,3\}.[0-9]\{1,3\}.[0-9]\{1,3\}.[0-9]\{1,3\}\([ \t]\+server2.com\)/' + server2 + '\1/g" /etc/hosts'
  cmd3 = 'sed -i "s/[0-9]\{1,3\}.[0-9]\{1,3\}.[0-9]\{1,3\}.[0-9]\{1,3\}\([ \t]\+haserver.com\)/' + haserver + '\1/g" /etc/hosts'
  cmd5 = 'sed -i "s/[0-9]\{1,3\}.[0-9]\{1,3\}.[0-9]\{1,3\}.[0-9]\{1,3\}\([ \t]\+anotherserver.com\)/' + another_server + '\1/g" /etc/hosts'

  `#{cmd1}`
  `#{cmd2}`
  `#{cmd3}`
  `#{cmd5}`
end

def generate_keepalived_conf haserver, role
  puts "生成keepalived.conf"
  keepalived_conf_orig = 
  <<EOF
global_defs {
   notification_email {
     ewhine@localhost
   }
   notification_email_from Dehui Network
   smtp_server localhost
   smtp_connect_timeout 30
   router_id LVS_DEVEL
}
vrrp_instance VI_1 {
    state BACKUP
    interface eth0
    virtual_router_id 51i
    priority priority_vi_1
    nopreempt
    notify_master /etc/keepalived/notify_master.sh
    notify_backup /etc/keepalived/notify_backup.sh
    advert_int 1
    authentication {
      auth_type PASS
      auth_pass 1111
    }
    virtual_ipaddress {
      vip_vi_1
    }
}

########Virtual Server########
virtual_server vip_vi_1 80 {
  delay_loop 6
  lb_algo rr
  lb_kind DR
  # nat_mask  255.255.255.0 # if you use linux kernel 2.2, set this line
  persistence_timeout 50
  protocol TCP
  real_server server1.com 80 {
    weight 1
    TCP_CHECK {
      connect_timeout 3
      nb_get_retry 3
      delay_before_retry 3
    }
  }
  real_server server2.com 80 {
    weight 3
    TCP_CHECK {
      connect_timeout 3
      nb_get_retry 3
      delay_before_retry 3
    }
  }
}
EOF

  case role
  when "master"
    priority_vi_1 = "250"
  when "slave"
    priority_vi_1 = "150"
  end

  keepalived_conf = keepalived_conf_orig.gsub('vip_vi_1', haserver).gsub('priority_vi_1', priority_vi_1)

  File.open('/etc/keepalived/keepalived.conf', 'w') do |f|
    f.write(keepalived_conf)
  end

  puts "生成notify_master.sh"
  notify_master_script =
  <<EOF
#!/bin/bash
echo [`date`]"作为master启动" >> /home/ewhine/keepalived.log
/home/ewhine/redis-2.8.12/src/redis-cli  slaveof NO ONE
EOF

  File.open('/etc/keepalived/notify_master.sh', 'w') do |f|
    f.write(notify_master_script)
  end
  `chmod +x /etc/keepalived/notify_master.sh`

  puts "生成notify_backup.sh"
  notify_backup_script =
  <<EOF
#!/bin/bash
echo [`date`]"作为backup启动" >> /home/ewhine/keepalived.log
du /home/ewhine/deploy/ewhine_NB/efiles  >> /dev/null &
echo "flush_all" | nc localhost 11211 -q5
/home/ewhine/redis-2.8.12/src/redis-cli  slaveof anotherserver.com 6379
EOF
  File.open('/etc/keepalived/notify_backup.sh', 'w') do |f|
    f.write(notify_backup_script)
  end
  `chmod +x /etc/keepalived/notify_master.sh`


  puts "生成check_local.sh"
  # FIXME: local_web_url不应该写死！
  check_local_script = 
  <<EOF
#!/bin/bash
pidfile=/var/lock/subsys/`basename $0`.pid
if [ -f $pidfile ] && [ -e /proc/`cat $pidfile` ] ; then
  exit 1
fi
trap "rm -fr $pidfile ; exit 0" 1 2 3 15
echo $$ > $pidfile

maxfails=3
fails=0
success=0

local_web_url="http://127.0.0.1:8016/accounts/sign_in"
gw=`ip route|grep "default via" | sed "s/default via \([0-9]\{1,3\}.[0-9]\{1,3\}.[0-9]\{1,3\}.[0-9]\{1,3\}\) dev .*/\1/g"`

while [ 1 ]
do
  # 如果rails服务宕掉
  /usr/bin/wget --timeout=3 --tries=1 $local_web_url -q -O /dev/null
  if [ $? -ne 0 ] ; then
    let fails=$[$fails+1]
    success=0
  else
    fails=0
    let success=$[$success+1]
  fi

  # 如果无法ping通网关
  result=`ping -c3 $gw|grep ttl`
  if [ -z "$result" ] ; then
    let fails=$[$fails+1]
    success=0
  eles
    fails=0
    let success=$[$success+1]
  fi

  if [ $fails -ge $maxfails ] ; then
    fails=0
    success=0

    #check keepalived is running ? try to stop it
    service keepalived status | grep running
    if [ $? -eq 0 ] ; then
      logger -is "local service fails $maxfails times ... try to stop keepalived."
      service keepalived stop 2>&1 | logger
    fi
  fi

  if [ $success -gt $maxfails ] ; then
    #check keepalived is stopped ? try to start it
    service keepalived status | grep stopped
    if [ $? -eq 0 ] ; then
      logger -is "service changes normal, try to start keepalived ."
      service keepalived start
    fi
    success=0
  fi
  sleep 3
done
EOF
  File.open('/etc/keepalived/check_local.sh', 'w') do |f|
    f.write(check_local_script)
  end
  `chmod +x /etc/keepalived/check_local.sh`
end


def configure_mysql_replication server1, server2, role
  puts "设置server_id"
  server_id = 
    case role
    when "master"
      "10"
    when "slave"
      "20"
    end

  sed_cmd = 'sed -i "s/\(server-id[ \t]*=[ \t]*\)[0-9]\+/\1' + server_id + '/g" /etc/mysql/my.cnf'
  `#{sed_cmd}`

  `service mysql restart`

  puts "设置mysql同步"
  if role == "master"
    `mysql -hserver1.com -uroot -pesns -e "stop slave; reset slave;"`
    `mysql -hserver2.com -uroot -pesns -e "stop slave; reset slave;"`
    `mysqldump -hserver1.com -uroot -pesns --add-drop-table --add-locks esns_production | mysql -hserver2.com -uroot -pesns esns_production`
    set_mysql server1, server2
    set_mysql server2, server1
  else
    # do nothing.
  end
end

def set_mysql origin, replica
  puts "配置从#{origin}到#{replica}的复制"
  cmd_orig =
  <<EOF
mysql -hreplica -uroot -pesns -e "stop slave; change master to  master_host='origin', master_user='slaveuser', master_password='dehui123', master_log_file='logfile', master_log_pos=logpos;start slave;"
EOF

  mysql_cmd = 'mysql -h' + origin + ' -uroot -pesns -e  "show master status\G"'
  res = `#{mysql_cmd}`
  logfile = (/File: (mariadb-bin.[0-9]+)/ =~ res; $1)
  logpos = (/Position: ([0-9]+)/ =~ res; $1)

  cmd = cmd_orig.gsub("logfile", logfile).gsub("logpos", logpos.to_s).gsub("replica", replica).gsub("origin", origin)
  `#{cmd}`
  # TODO:检查slave状态是否成功启动
end

def configure_glusterfs_client role, server1, server2
  # 生成glusterfs客户端配置文件
  puts "生成glusterfs客户端配置文件"
  glusterfs_vol =
  <<EOF
volume self
type protocol/client
option transport-type tcp
option remote-host 127.0.0.1
option remote-subvolume brick
end-volume

volume remote
type protocol/client
option transport-type tcp
option remote-host anotherserver.com
option remote-subvolume brick
end-volume

volume replicate
type cluster/replicate
subvolumes self remote
end-volume

volume writebehind
type performance/write-behind
option cache-size 512KB
option flush-behind on
subvolumes replicate
end-volume

volume cache
type performance/io-cache
option cache-size 1024MB
subvolumes writebehind
end-volume
EOF

  File.open("/etc/glusterfs/glusterfs.vol", "w") do |f|
    f.write(glusterfs_vol)
  end

end

def configure_memcached_ha
end

# 重启glusterfs-server keepalived，重启nginx，挂载虚拟分区
def restart_services1_and_mount
  `service glusterfs-server restart && /opt/nginx/sbin/nginx -s reload && umount /home/ewhine/deploy/ewhine_NB/efiles  && service keepalived restart && mount -a`
end

# 将rails项目中的mysql/redis/sidekiq配置文件的host修改为虚拟IP并重启rails和sidekiq
def change_rails_config
end


case
when ["--help", "-h"].include?(ARGV.first)
  put_usage
when ARGV.size != 4
  puts "参数数量不正确。"
  put_usage
when ARGV.first != "master" && ARGV.first != "slave"
  puts "角色不正确，应为master或slave"
else
  puts "开始配置"

  role, server1, server2, haserver = ARGV

  # 设置host表
  set_hosts server1, server2, haserver, role

  # 生成keepalived配置文件
  generate_keepalived_conf haserver, role

  # 修改mysql的server_id, 配置replication
  configure_mysql_replication server1, server2, role

  configure_memcached_ha

  # 配置glusterfs客户端配置文件
  configure_glusterfs_client role, server1, server2


  # 重启glusterfs keepalived,挂载虚拟分区
  restart_services1_and_mount

  # 将rails项目中的mysql/redis/sidekiq配置文件的host修改为虚拟IP并重启rails和sidekiq
  # change_rails_config

  puts "配置完毕。"
end
